@@ -18,6 +18,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:bluetooth-chooser-button
+ * @short_description: a Bluetooth chooser button
+ * @stability: Stable
+ * @include: bluetooth-plugin.h
+ *
+ * A button used to select Bluetooth devices which will pop-up a
+ * #BluetoothChooser widget inside a dialogue when clicked.
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -191,7 +201,7 @@ bluetooth_chooser_button_clicked (GtkButton *widget)
 	parent = gtk_widget_get_toplevel (GTK_WIDGET (button));
 	//FIXME title
 	button->dialog = gtk_dialog_new_with_buttons("", GTK_WINDOW (parent),
-						     GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
+						     GTK_DIALOG_MODAL,
 						     GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
 						     GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
 	g_signal_connect (button->dialog, "response",
@@ -379,7 +389,9 @@ bluetooth_chooser_button_init (BluetoothChooserButton *button)
 /**
  * bluetooth_chooser_button_new:
  *
- * Return value: a #BluetoothChooserButton
+ * Returns a new #BluetoothChooserButton widget.
+ *
+ * Return value: a #BluetoothChooserButton widget.
  **/
 GtkWidget *
 bluetooth_chooser_button_new (void)
@@ -393,6 +405,8 @@ bluetooth_chooser_button_new (void)
  * bluetooth_chooser_button_available:
  * @button: a #BluetoothChooserButton
  *
+ * Returns whether there is a powered Bluetooth adapter.
+ *
  * Return value: %TRUE if there is a powered Bluetooth adapter available, and the button should be sensitive.
  **/
 gboolean
