@@ -26,7 +26,7 @@
 #include <config.h>
 #endif
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
 #include <dbus/dbus-glib.h>
@@ -89,7 +89,7 @@ static void discoverable_changed_cb(GtkWidget *button, gpointer user_data)
 	/* Note that this would be broken if we showed more than
 	 * one adapter, but we don't care */
 	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (button), FALSE);
-	bluetooth_client_set_discoverable (client, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+	bluetooth_client_set_discoverable (client, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)), 0);
 }
 
 static void name_callback(GtkWidget *editable, gpointer user_data)
@@ -360,7 +360,7 @@ static void create_adapter(adapter_data *adapter)
 
 	adapter->devices_table = table;
 
-	buttonbox = gtk_vbutton_box_new();
+	buttonbox = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox),
 						GTK_BUTTONBOX_START);
 	gtk_box_set_spacing(GTK_BOX(buttonbox), 6);
@@ -698,7 +698,7 @@ create_no_adapter_page (GtkNotebook *notebook, const char *filename)
 		path = g_build_filename (PKGDATADIR, filename, NULL);
 		if (gtk_builder_add_from_file (xml, path, NULL) == 0) {
 			g_free (path);
-			g_warning ("Failed to load properties-no-adapter.ui");
+			g_warning ("Failed to load '%s'", filename);
 			return;
 		}
 		g_free (path);
