@@ -42,8 +42,11 @@
 
 #define AGENT_PATH "/org/bluez/agent/sendto"
 
+#define RESPONSE_RETRY 1
+
 static DBusGConnection *conn = NULL;
 static ObexAgent *agent = NULL;
+static DBusGProxy *client_proxy = NULL;
 
 static GtkWidget *dialog;
 static GtkWidget *label_from;
@@ -66,6 +69,21 @@ static int file_index = 0;
 static gint64 first_update = 0;
 static gint64 last_update = 0;
 
+static void send_notify(DBusGProxy *proxy, DBusGProxyCall *call, void *user_data);
+
+/* Agent callbacks */
+static gboolean release_callback(DBusGMethodInvocation *context, gpointer user_data);
+static gboolean request_callback(DBusGMethodInvocation *context, DBusGProxy *transfer, gpointer user_data);
+static gboolean progress_callback(DBusGMethodInvocation *context,
+				  DBusGProxy *transfer,
+				  guint64 transferred,
+				  gpointer user_data);
+static gboolean complete_callback(DBusGMethodInvocation *context, DBusGProxy *transfer, gpointer user_data);
+static gboolean error_callback(DBusGMethodInvocation *context,
+			       DBusGProxy *transfer,
+			       const char *message,
+			       gpointer user_data);
+
 static void value_free(GValue *value)
 {
 	g_value_unset(value);
@@ -73,7 +91,7 @@ static void value_free(GValue *value)
 }
 
 static GHashTable *
-new_hash_table_for_address (const char *address)
+send_files (void)
 {
 	GHashTable *hash;
 	GValue *value;
@@ -83,12 +101,34 @@ new_hash_table_for_address (const char *address)
 
 	value = g_new0(GValue, 1);
 	g_value_init(value, G_TYPE_STRING);
-	g_value_set_string(value, address);
+	g_value_set_string(value, option_device);
 	g_hash_table_insert(hash, g_strdup("Destination"), value);
 
+	dbus_g_proxy_begin_call(client_proxy, "SendFiles",
+				send_notify, NULL, NULL,
+				dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), hash,
+				G_TYPE_STRV, option_files,
+				DBUS_TYPE_G_OBJECT_PATH, AGENT_PATH,
+				G_TYPE_INVALID);
+
 	return hash;
 }
 
+static void
+setup_agent (void)
+{
+	if (agent == NULL)
+		agent = obex_agent_new();
+
+	obex_agent_set_release_func(agent, release_callback, NULL);
+	obex_agent_set_request_func(agent, request_callback, NULL);
+	obex_agent_set_progress_func(agent, progress_callback, NULL);
+	obex_agent_set_complete_func(agent, complete_callback, NULL);
+	obex_agent_set_error_func(agent, error_callback, NULL);
+
+	obex_agent_setup(agent, AGENT_PATH);
+}
+
 static gchar *filename_to_path(const gchar *filename)
 {
 	GFile *file;
@@ -149,43 +189,6 @@ static gchar *format_time(gint seconds)
 				"approximately %'d hours", hours), hours);
 }
 
-typedef struct _ResponseData ResponseData;
-struct _ResponseData {
-	gint response_id;
-};
-
-static GtkWidget *
-get_widget_for_response (GtkDialog *dialog,
-			 int response_id)
-{
-	GList *children;
-	GList *tmp_list;
-
-	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
-
-	children = gtk_container_get_children (GTK_CONTAINER (gtk_dialog_get_action_area (dialog)));
-
-	tmp_list = children;
-	while (tmp_list != NULL) {
-		GtkWidget *widget = tmp_list->data;
-		ResponseData *rd = g_object_get_data (G_OBJECT (widget),
-						      "gtk-dialog-response-data");
-
-		if (rd && rd->response_id == response_id) {
-			g_list_free (children);
-			return widget;
-		}
-
-		tmp_list = g_list_next (tmp_list);
-	}
-
-	g_list_free (children);
-
-	return NULL;
-
-
-}
-
 static void
 set_response_visible (GtkDialog *dialog,
 		      int response_id,
@@ -193,7 +196,7 @@ set_response_visible (GtkDialog *dialog,
 {
 	GtkWidget *widget;
 
-	widget = get_widget_for_response (dialog, response_id);
+	widget = gtk_dialog_get_widget_for_response (dialog, response_id);
 	gtk_widget_set_no_show_all (widget, TRUE);
 	gtk_widget_set_visible (widget, visible);
 }
@@ -201,14 +204,31 @@ set_response_visible (GtkDialog *dialog,
 static void response_callback(GtkWidget *dialog,
 					gint response, gpointer user_data)
 {
+	if (response == RESPONSE_RETRY) {
+		setup_agent ();
+
+		/* Reset buttons */
+		set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE, FALSE);
+		set_response_visible (GTK_DIALOG (dialog), RESPONSE_RETRY, FALSE);
+		set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL, TRUE);
+
+		/* Reset status and progress bar */
+		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progress),
+					  _("Connecting..."));
+		gtk_label_set_text (GTK_LABEL (label_status), "");
+		gtk_widget_hide (image_status);
+		send_files ();
+		return;
+	}
+
 	if (current_transfer != NULL) {
 		obex_agent_set_error_func(agent, NULL, NULL);
 		dbus_g_proxy_call_no_reply (current_transfer, "Cancel", G_TYPE_INVALID);
 		g_object_unref (current_transfer);
 		current_transfer = NULL;
 	}
-	gtk_widget_destroy(dialog);
 
+	gtk_widget_destroy(dialog);
 	gtk_main_quit();
 }
 
@@ -227,11 +247,13 @@ static void create_window(void)
 	gchar *text;
 
 	dialog = gtk_dialog_new_with_buttons(_("File Transfer"), NULL,
-				GTK_DIALOG_NO_SEPARATOR,
+				0,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+				_("_Retry"), RESPONSE_RETRY,
 				NULL);
 	set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE, FALSE);
+	set_response_visible (GTK_DIALOG (dialog), RESPONSE_RETRY, FALSE);
 	gtk_window_set_type_hint(GTK_WINDOW(dialog),
 						GDK_WINDOW_TYPE_HINT_NORMAL);
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
@@ -241,7 +263,8 @@ static void create_window(void)
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_box_set_spacing(GTK_BOX(vbox), 6);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
+	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+	                   vbox);
 
 	label = gtk_label_new(NULL);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -326,7 +349,7 @@ static gchar *get_error_message(GError *error)
 
 	if (dbus_g_error_has_name(error, OPENOBEX_CONNECTION_FAILED) == TRUE &&
 					is_palm_device(option_device)) {
-		message = g_strdup(_("Make sure that remote device "
+		message = g_strdup(_("Make sure that the remote device "
 					"is switched on and that it "
 					"accepts Bluetooth connections"));
 		goto done;
@@ -536,6 +559,8 @@ static gboolean release_callback(DBusGMethodInvocation *context,
 {
 	dbus_g_method_return(context);
 
+	agent = NULL;
+
 	gtk_label_set_markup(GTK_LABEL(label_status), NULL);
 
 	gtk_widget_destroy(dialog);
@@ -550,7 +575,20 @@ static gboolean error_callback(DBusGMethodInvocation *context,
 			       const char *message,
 			       gpointer user_data)
 {
-	g_message ("Error message: %s", message);
+	gtk_widget_show (image_status);
+	gtk_label_set_markup(GTK_LABEL(label_status), message);
+
+	set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL, TRUE);
+	set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE, FALSE);
+	set_response_visible (GTK_DIALOG (dialog), RESPONSE_RETRY, TRUE);
+
+	g_object_unref (current_transfer);
+	current_transfer = NULL;
+
+	if (agent != NULL) {
+		obex_agent_set_release_func(agent, NULL, NULL);
+		agent = NULL;
+	}
 
 	dbus_g_method_return(context);
 
@@ -571,8 +609,9 @@ static void send_notify(DBusGProxy *proxy,
 		gtk_label_set_markup(GTK_LABEL(label_status), message);
 		g_free (message);
 
-		set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL, FALSE);
-		set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE, TRUE);
+		set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL, TRUE);
+		set_response_visible (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE, FALSE);
+		set_response_visible (GTK_DIALOG (dialog), RESPONSE_RETRY, TRUE);
 		return;
 	}
 
@@ -595,12 +634,12 @@ select_device_changed(BluetoothChooser *sel,
 static char *
 show_browse_dialog (char **device_name)
 {
-	GtkWidget *dialog, *selector, *send_button, *image;
+	GtkWidget *dialog, *selector, *send_button, *image, *content_area;
 	char *bdaddr;
 	int response_id;
 
 	dialog = gtk_dialog_new_with_buttons(_("Select Device to Send To"), NULL,
-					     GTK_DIALOG_NO_SEPARATOR,
+					     0,
 					     GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
 					     NULL);
 	send_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("Send _To"), GTK_RESPONSE_ACCEPT);
@@ -611,7 +650,8 @@ show_browse_dialog (char **device_name)
 	gtk_window_set_default_size(GTK_WINDOW(dialog), 480, 400);
 
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 
 	selector = bluetooth_chooser_new("Select device to send files to");
 	gtk_container_set_border_width(GTK_CONTAINER(selector), 5);
@@ -623,7 +663,7 @@ show_browse_dialog (char **device_name)
 		     NULL);
 	g_signal_connect(selector, "selected-device-changed",
 			 G_CALLBACK(select_device_changed), dialog);
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), selector);
+	gtk_container_add (GTK_CONTAINER (content_area), selector);
 	bluetooth_chooser_start_discovery (BLUETOOTH_CHOOSER (selector));
 
 	bdaddr = NULL;
@@ -685,7 +725,6 @@ static GOptionEntry options[] = {
 
 int main(int argc, char *argv[])
 {
-	DBusGProxy *proxy;
 	GError *error = NULL;
 	int i;
 
@@ -792,30 +831,16 @@ int main(int argc, char *argv[])
 
 	create_window();
 
-	proxy = dbus_g_proxy_new_for_name(conn, "org.openobex.client",
+	client_proxy = dbus_g_proxy_new_for_name(conn, "org.openobex.client",
 					"/", "org.openobex.Client");
 
-	agent = obex_agent_new();
+	setup_agent ();
 
-	obex_agent_set_release_func(agent, release_callback, NULL);
-	obex_agent_set_request_func(agent, request_callback, NULL);
-	obex_agent_set_progress_func(agent, progress_callback, NULL);
-	obex_agent_set_complete_func(agent, complete_callback, NULL);
-	obex_agent_set_error_func(agent, error_callback, NULL);
-
-	obex_agent_setup(agent, AGENT_PATH);
-
-	dbus_g_proxy_begin_call(proxy, "SendFiles",
-				send_notify, NULL, NULL,
-				dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE),
-				new_hash_table_for_address (option_device),
-				G_TYPE_STRV, option_files,
-				DBUS_TYPE_G_OBJECT_PATH, AGENT_PATH,
-				G_TYPE_INVALID);
+	send_files ();
 
 	gtk_main();
 
-	g_object_unref(proxy);
+	g_object_unref(client_proxy);
 
 	dbus_g_connection_unref(conn);
 
