@@ -31,7 +31,6 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <dbus/dbus-glib.h>
-#include <unique/uniqueapp.h>
 
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
@@ -82,6 +81,8 @@ static char *pincode = NULL;
 static GtkBuilder *builder = NULL;
 
 static GtkAssistant *window_assistant = NULL;
+static GtkWidget *button_close = NULL;
+static GtkWidget *button_cancel = NULL;
 static GtkWidget *page_search = NULL;
 static GtkWidget *page_connecting = NULL;
 static GtkWidget *page_setup = NULL;
@@ -411,88 +412,15 @@ close_callback (GtkWidget *assistant,
 	gtk_main_quit();
 }
 
-/* HACK, to access the GtkAssistant buttons */
-struct RealGtkAssistant
-{
-	GtkWindow  parent;
-
-	GtkWidget *cancel;
-	GtkWidget *forward;
-	GtkWidget *back;
-	GtkWidget *apply;
-	GtkWidget *close;
-	GtkWidget *last;
-
-	/*< private >*/
-	GtkAssistantPrivate *priv;
-};
-typedef struct RealGtkAssistant RealGtkAssistant;
-
-static gboolean
-prepare_idle_cb (gpointer data)
-{
-	RealGtkAssistant *assistant = (RealGtkAssistant *) window_assistant;
-	gint page;
-
-	page = gtk_assistant_get_current_page (GTK_ASSISTANT (window_assistant));
-	if (page == PAGE_FAILURE) {
-		gtk_widget_hide (assistant->cancel);
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_show (assistant->close);
-		gtk_widget_set_sensitive (assistant->close, TRUE);
-	}
-	if (page == PAGE_CONNECTING) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->close);
-		gtk_widget_show (assistant->cancel);
-		gtk_widget_set_sensitive (assistant->cancel, TRUE);
-	}
-	if (page == PAGE_SETUP) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->close);
-		gtk_widget_show (assistant->cancel);
-		gtk_widget_set_sensitive (assistant->cancel, TRUE);
-	}
-	if (page == PAGE_SSP_SETUP) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->close);
-		if (display_called == FALSE) {
-			gtk_widget_hide (assistant->cancel);
-		} else {
-			gtk_widget_show (assistant->cancel);
-			gtk_widget_set_sensitive (assistant->cancel, TRUE);
-		}
-	}
-	if (page == PAGE_FINISHING) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->cancel);
-		gtk_widget_show (assistant->close);
-		gtk_widget_set_sensitive (assistant->close, FALSE);
-	}
-	return FALSE;
-}
-
 void prepare_callback (GtkWidget *assistant,
 		       GtkWidget *page,
 		       gpointer data)
 {
 	gboolean complete = TRUE;
 
+	gtk_widget_hide (button_close);
+	gtk_widget_hide (button_cancel);
+
 	if (page == page_search) {
 		complete = set_page_search_complete ();
 		bluetooth_chooser_start_discovery(selector);
@@ -509,11 +437,13 @@ void prepare_callback (GtkWidget *assistant,
 
 		/* translators:
 		 * The '%s' is the device name, for example:
-		 * Connecting to 'Sony Bluetooth Headset' now...
+		 * Connecting to 'Sony Bluetooth Headset'...
 		 */
 		text = g_strdup_printf (_("Connecting to '%s'..."), target_name);
 		gtk_label_set_text (GTK_LABEL (label_connecting), text);
 		g_free (text);
+
+		gtk_widget_show (button_cancel);
 	} else {
 		gtk_spinner_stop (GTK_SPINNER (spinner_connecting));
 	}
@@ -557,6 +487,8 @@ void prepare_callback (GtkWidget *assistant,
 		} else {
 			g_assert_not_reached ();
 		}
+
+		gtk_widget_show (button_cancel);
 	}
 
 	if (page == page_finishing) {
@@ -573,6 +505,8 @@ void prepare_callback (GtkWidget *assistant,
 		text = g_strdup_printf (_("Please wait while finishing setup on device '%s'..."), target_name);
 		gtk_label_set_text (GTK_LABEL (label_finishing), text);
 		g_free (text);
+
+		gtk_widget_show (button_close);
 	} else {
 		gtk_spinner_stop (GTK_SPINNER (spinner_finishing));
 	}
@@ -617,15 +551,20 @@ void prepare_callback (GtkWidget *assistant,
 	if (page == page_failure) {
 		complete = FALSE;
 		gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("restart_button"));
+		gtk_widget_show (button_close);
 	} else {
 		if (gtk_widget_get_parent (W("restart_button")) != NULL)
 			gtk_assistant_remove_action_widget (GTK_ASSISTANT (assistant), W("restart_button"));
 	}
 
-	if (page == page_ssp_setup && display_called == FALSE) {
-		complete = FALSE;
-		gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("matches_button"));
-		gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("does_not_match_button"));
+	if (page == page_ssp_setup) {
+		if (display_called == FALSE) {
+			complete = FALSE;
+			gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("matches_button"));
+			gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("does_not_match_button"));
+		} else {
+			gtk_widget_show (button_cancel);
+		}
 	} else {
 		if (gtk_widget_get_parent (W("does_not_match_button")) != NULL)
 			gtk_assistant_remove_action_widget (GTK_ASSISTANT (assistant), W("does_not_match_button"));
@@ -635,11 +574,6 @@ void prepare_callback (GtkWidget *assistant,
 
 	gtk_assistant_set_page_complete (GTK_ASSISTANT(assistant),
 					 page, complete);
-
-	/* HACK to allow hiding/showing the buttons
-	 * instead of relying on the GtkAssistant doing that
-	 * for us */
-	g_idle_add (prepare_idle_cb, NULL);
 }
 
 static gboolean
@@ -673,8 +607,8 @@ entry_custom_event (GtkWidget *entry, GdkEventKey *event)
 	if (event->length == 0)
 		return FALSE;
 
-	if ((event->keyval >= GDK_0 && event->keyval <= GDK_9) ||
-	    (event->keyval >= GDK_KP_0 && event->keyval <= GDK_KP_9))
+	if ((event->keyval >= GDK_KEY_0 && event->keyval <= GDK_KEY_9) ||
+	    (event->keyval >= GDK_KEY_KP_0 && event->keyval <= GDK_KEY_KP_9))
 		return FALSE;
 
 	return TRUE;
@@ -894,6 +828,14 @@ create_wizard (void)
 
 	gtk_assistant_set_forward_page_func (assistant, page_func, NULL, NULL);
 
+	/* The 2 custom buttons */
+	button_close = W("close_button");
+	button_cancel = W("cancel_button");
+	gtk_assistant_add_action_widget (assistant, button_close);
+	gtk_assistant_add_action_widget (assistant, button_cancel);
+	gtk_widget_hide (button_close);
+	gtk_widget_hide (button_cancel);
+
 	/* Intro page */
 	combo = gtk_combo_box_new();
 
@@ -985,16 +927,11 @@ create_wizard (void)
 	return assistant;
 }
 
-static UniqueResponse
-message_received_cb (UniqueApp         *app,
-                     int                command,
-                     UniqueMessageData *message_data,
-                     guint              time_,
-                     gpointer           user_data)
+static void
+activate_cb (GApplication      *app,
+	     gpointer           user_data)
 {
-        gtk_window_present (GTK_WINDOW (user_data));
-
-        return UNIQUE_RESPONSE_OK;
+	gtk_window_present_with_time (GTK_WINDOW (user_data), GDK_CURRENT_TIME);
 }
 
 static GOptionEntry options[] = {
@@ -1003,7 +940,7 @@ static GOptionEntry options[] = {
 
 int main (int argc, char **argv)
 {
-	UniqueApp *app;
+	GtkApplication *app;
 	GError *error = NULL;
 
 	bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
@@ -1021,10 +958,16 @@ int main (int argc, char **argv)
 		return 1;
 	}
 
-	app = unique_app_new ("org.gnome.Bluetooth.wizard", NULL);
-	if (unique_app_is_running (app)) {
+	app = gtk_application_new ("org.gnome.Bluetooth.wizard", G_APPLICATION_FLAGS_NONE);
+	if (g_application_register (G_APPLICATION (app), NULL, &error) == FALSE) {
+		g_warning ("Could not register application: %s", error->message);
+		g_error_free (error);
+		return 1;
+	}
+
+	if (g_application_get_is_remote (G_APPLICATION (app))) {
+		g_application_activate (G_APPLICATION (app));
 		gdk_notify_startup_complete ();
-		unique_app_send_message (app, UNIQUE_ACTIVATE, NULL);
 		return 0;
 	}
 
@@ -1035,6 +978,7 @@ int main (int argc, char **argv)
 	client = bluetooth_client_new();
 
 	agent = bluetooth_agent_new();
+	g_object_add_weak_pointer (G_OBJECT (agent), (gpointer *) (&agent));
 
 	bluetooth_agent_set_pincode_func(agent, pincode_callback, NULL);
 	bluetooth_agent_set_display_func(agent, display_callback, NULL);
@@ -1048,15 +992,18 @@ int main (int argc, char **argv)
 	window_assistant = create_wizard();
 	if (window_assistant == NULL)
 		return 1;
+	gtk_application_add_window (app,
+				    GTK_WINDOW (window_assistant));
 
-	g_signal_connect (app, "message-received",
-			  G_CALLBACK (message_received_cb), window_assistant);
+	g_signal_connect (app, "activate",
+			  G_CALLBACK (activate_cb), window_assistant);
 
-	gtk_main();
+	g_application_run (G_APPLICATION (app), argc, argv);
 
 	bluetooth_plugin_manager_cleanup ();
 
-	g_object_unref(agent);
+	if (agent != NULL)
+		g_object_unref (agent);
 
 	g_object_unref(client);
 
